@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import json
 from pathlib import Path
 from string import Template
@@ -50,31 +52,49 @@ $cls_rows            \midrule
 class Evaluation:
     def __init__(
         self,
+        dataset_name: str,
+        trajs_ids: List[str],
+        y_true,
+        y_pred,
         model_summary: dict,
-        data: Data,
-        predictions: List[Any],
     ):
-        self.dataset = data.dataset
-        self.trajs = data.trajs
-        self.y_true = data.labels
-        self.y_pred = predictions
+        self.dataset_name = dataset_name
+        self.traj_ids = trajs_ids
+        self.y_true = y_true if not isinstance(y_true, np.ndarray) else y_true.tolist()
+        self.y_pred = y_pred if not isinstance(y_pred, np.ndarray) else y_pred.tolist()
         self.model_summary = model_summary
         self.classes = list(set(self.y_true))
+        self.classes.sort()
 
         self._confusion_matrix = confusion_matrix(
             self.y_true, self.y_pred, labels=self.classes
+        ).T
+        pre, rec, f_sc, sup = precision_recall_fscore_support(
+            self.y_true, self.y_pred, labels=self.classes, zero_division=0
         )
-        (
-            self.precision,
-            self.recall,
-            self.f_score,
-            self.support,
-        ) = precision_recall_fscore_support(
-            self.y_true, self.y_pred, labels=self.classes
-        )
+        self.precision = np.asarray(pre)
+        self.recall = np.asarray(rec)
+        self.f_score = np.asarray(f_sc)
+        self.support = np.asarray(sup)
 
         self.acc_overall = accuracy_score(self.y_true, self.y_pred, normalize=True)
-        self.f1_score = f1_score(self.y_true, self.y_pred, average="weighted")
+        self.f1_score = f1_score(
+            self.y_true, self.y_pred, average="macro", zero_division=0
+        )
+
+    @staticmethod
+    def from_data(
+        data: Data,
+        predictions: List[Any],
+        model_summary: dict,
+    ) -> Evaluation:
+        return Evaluation(
+            dataset_name=data.dataset.name,
+            trajs_ids=[traj.traj_id for traj in data.trajs if traj.traj_id is not None],
+            y_true=data.labels,
+            y_pred=predictions,
+            model_summary=model_summary,
+        )
 
     def _conf_matrix_perc(self) -> np.ndarray:
         c_matrix = self._confusion_matrix.astype("float")
@@ -89,28 +109,70 @@ class Evaluation:
         # Normalize the confusion matrix by columns
         c_matrix = self._conf_matrix_perc()
 
-        # Round the values
-        # cm = np.round(cm, decimals=2)
         classes = list(self.classes) + ["precision"]
-        col_width = 12
+        col_width = max(5, max(map(len, map(str, classes[:-1])))) + 2
 
-        print(*[f"{c:<12}".format() for c in classes], sep="")
+        print(*[f"{c:<{col_width}}".format() for c in classes], sep="")
         print(MAIN_SEP * col_width * (len(classes)))
         for i, row in enumerate(c_matrix):
-            # get the precision
             row = np.append(row, self.precision[i])
-            print(*[f"{round(c * 100, 2):<12}" for c in row], sep="")
+            print(*[f"{round(c * 100, 2):<{col_width}}" for c in row], sep="")
         print(SUB_SEP * col_width * (len(classes)))
         print(
-            *[f"{round(rc * 100, 2):<12}" for rc in self.recall],
+            *[f"{round(rc * 100, 2):<{col_width}}" for rc in self.recall],
             sep="",
         )
 
     def show(self):
         """Show the evaluation results."""
+        self._show_general_stats()
         self._show_confusion_matrix()
 
-    def save(self, file_name: str) -> Path:
+    def _show_general_stats(self):
+        """Show the general statistics."""
+        print("\nGeneral statistics:\n")
+        print(f"Accuracy: {self.acc_overall:.3f}")
+        print(f"F1-score: {self.f1_score:.3f}")
+        print(f"Mean precision: {self.precision.mean():.3f}")
+        print(f"Mean recall: {self.recall.mean():.3f}")
+
+    @staticmethod
+    def load(file_name: str) -> Evaluation:
+        """Loads an evaluation from a file.
+
+        Parameters
+        ----------
+        file_name : str
+            The name of the file to load the evaluation from. It
+            must end with '.json'.
+
+        Returns
+        -------
+        Evaluation
+            The loaded evaluation.
+        """
+
+        if not file_name.endswith(".json"):
+            raise ValueError("file_name extension must be '.json'")
+
+        with open(file_name, "r", encoding="utf-8") as data_fd:
+            data = json.load(data_fd)
+
+        assert len(data["indices"]) == len(data["y_pred"])
+        ds_name = data["dataset_name"]
+        indices = data["indices"]
+        y_pred = data["y_pred"]
+        y_true = data["y_true"]
+        summary = data["model_summary"]
+        return Evaluation(
+            dataset_name=ds_name,
+            trajs_ids=indices,
+            y_pred=y_pred,
+            y_true=y_true,
+            model_summary=summary,
+        )
+
+    def save(self, file_name: str):
         """Save the evaluation to a file.
 
         Parameters
@@ -118,41 +180,39 @@ class Evaluation:
         file_name : str
             The name of the file to save the evaluation to. It
             must end with '.json'.
-
-        Returns
-        -------
-        Path
-            The path to the saved file.
         """
 
         if not file_name.endswith(".json"):
             raise ValueError("file_name extension must be '.json'")
 
         data = {
-            "indices": [
-                int(traj.traj_id) for traj in self.trajs if traj.traj_id is not None
-            ],
-            "predictions": self.y_pred,
+            "dataset_name": self.dataset_name,
+            "indices": self.traj_ids,
+            "y_pred": self.y_pred,
+            "y_true": self.y_true,
+            "classes": self.classes,
             "model_summary": self.model_summary,
         }
 
-        assert len(data["indices"]) == len(data["predictions"])
+        assert len(data["indices"]) == len(data["y_pred"])
 
-        file_path = _get_path(config.DS_EVALS_DIR, self.dataset.name) / file_name
+        if "/" in file_name:
+            dir_name = "/".join(file_name.split("/")[:-1])
+            dir_path = Path(dir_name)
+            dir_path.mkdir(parents=True, exist_ok=True)
 
-        with open(file_path, "w", encoding="utf-8") as data_fd:
+        with open(file_name, "w", encoding="utf-8") as data_fd:
             json.dump(data, data_fd, ensure_ascii=False, indent=4)
 
-        return file_path
-
     def to_markdown(self) -> str:
         """Evaluation summary in markdown style."""
-        model_name = self.model_summary.pop("name")
+        summary = self.model_summary.copy()
+        model_name = summary.pop("name")
         ans = "# Evaluation results\n\n"
-        ans += f"**Dataset:** {self.dataset.name} \\\n"
+        ans += f"**Dataset:** {self.dataset_name} \\\n"
         ans += f"**Model:** {model_name}\n"
         ans += "\n## Model Summary\n\n"
-        for param, val in self.model_summary.items():
+        for param, val in summary.items():
             ans += f"- `{param} = {val}`\n"
         ans += "\n## Confusion Matrix\n\n"
 
@@ -180,10 +240,10 @@ class Evaluation:
 
     def to_latex(self) -> str:
         """Evaluation summary in latex style."""
-        model_name = self.model_summary.pop("name")
+        summary = self.model_summary.copy()
+        model_name = summary.pop("name")
         model_name = " ".join([val.title() for val in model_name.split("_")])
-        ans = ""
-        cls_rows = ""
+        ans = cls_rows = ""
         c_matrix = self._conf_matrix_perc()
         classes = [cls.replace("_", "\\_") for cls in self.classes]
 
@@ -198,7 +258,7 @@ class Evaluation:
             cls_rows += row
 
         ans += LATEX_CM_TEMPLATE.substitute(
-            caption=f"Confusion matrix for {model_name}. Dataset: {self.dataset.name}",
+            caption=f"Confusion matrix for {model_name}. Dataset: {self.dataset_name}",
             model_name=model_name,
             c_cols="c" * len(classes),
             c_line_top=str(len(classes) + 1),
