@@ -17,14 +17,14 @@ class RandomForestModel(Model):
     def __init__(self, featurizer: featurizers.Featurizer, **kwargs):
         super().__init__(NAME)
         self.featurizer = featurizer
-        self.rfc = RandomForestClassifier(**kwargs)
+        self.model = RandomForestClassifier(**kwargs)
         self.grid: GridSearchCV
         self.set_summary(**kwargs)
 
     def train(self, data: Data, cross_validation: int = 0):
         self.set_summary(cross_validation=cross_validation)
         x_data = data.featurize(self.featurizer)
-        self.grid = GridSearchCV(self.rfc, {}, cv=cross_validation, verbose=3)
+        self.grid = GridSearchCV(self.model, {}, cv=cross_validation, verbose=3)
         self.grid.fit(x_data, data.labels)
 
     def predict(self, data: Data) -> List[Any]:
