@@ -6,11 +6,12 @@ import numpy as np
 from sklearn.model_selection import KFold
 from sklearn.preprocessing import LabelEncoder
 from tensorflow import keras
-from tensorflow.keras.callbacks import ModelCheckpoint
 
+import pactus.config as cfg
 from pactus.dataset import Data
 from pactus.models import Model
-from pactus.models.transformer import build_model
+from pactus.models.evaluation import Evaluation
+from pactus.models.transformer import TransformerBlock, build_model
 
 NAME = "transformer_model"
 DEFAULT_OPTIMIZER = keras.optimizers.Adam(learning_rate=1e-2)
@@ -29,28 +30,33 @@ class TransformerModel(Model):
         ff_dim: int = 4,
         num_transformer_blocks: int = 2,
         mlp_units: Union[List[int], None] = None,
-        mlp_droput: float = 0.4,
-        droput: float = 0.25,
+        mlp_dropout: float = 0.4,
+        dropout: float = 0.25,
         loss="categorical_crossentropy",
         optimizer=None,
         metrics=None,
         max_traj_len: int = -1,
         skip_long_trajs: bool = False,
+        mask_value=cfg.MASK_VALUE,
+        name=NAME,
     ):
-        super().__init__(NAME)
+        super().__init__(name)
         self.head_size = head_size
         self.num_heads = num_heads
         self.ff_dim = ff_dim
         self.num_transformer_blocks = num_transformer_blocks
         self.mlp_units = [128] if mlp_units is None else mlp_units
-        self.mlp_dropout = mlp_droput
-        self.dropout = droput
+        self.mlp_dropout = mlp_dropout
+        self.dropout = dropout
         self.model: keras.Model
         self.loss = loss
         self.optimizer = DEFAULT_OPTIMIZER if optimizer is None else optimizer
         self.metrics = ["accuracy"] if metrics is None else metrics
         self.max_traj_len = max_traj_len
         self.skip_long_trajs = skip_long_trajs
+        self.mask_value = mask_value
+        self.encoder: Union[LabelEncoder, None] = None
+        self.labels: Union[List[Any], None] = None
         self.set_summary(
             head_size=self.head_size,
             num_heads=self.num_heads,
@@ -82,10 +88,12 @@ class TransformerModel(Model):
             validation_split=validation_split,
             batch_size=batch_size,
         )
-        x_train, y_train, mask = self._get_input_data(data)
-        n_classes = len(data.dataset.classes)
+        self.encoder = None
+        self.labels = data.labels
+        x_train, y_train = self._get_input_data(data)
+        n_classes = len(data.classes)
         input_shape = x_train.shape[1:]
-        callbacks = DEFAULT_CALLBACKS if callbacks is None else callbacks
+        callbacks = DEFAULT_CALLBACKS.copy() if callbacks is None else callbacks
         model_path = None
         if checkpoint is not None:
             callbacks.append(checkpoint)
@@ -95,9 +103,15 @@ class TransformerModel(Model):
 
         if cross_validation == 0:
             model = (
-                self._get_model(n_classes, input_shape, mask)
+                self._get_model(
+                    n_classes,
+                    input_shape,
+                    mask=self.mask_value,
+                )
                 if model_path is None
-                else keras.models.load_model(model_path)
+                else keras.models.load_model(
+                    model_path, custom_objects={"TransformerBlock": TransformerBlock}
+                )
             )
             model.fit(
                 x_train,
@@ -117,7 +131,11 @@ class TransformerModel(Model):
             for train_idxs, test_idxs in kfold.split(x_train, y_train):
                 x_train_fold = x_train[train_idxs]
                 y_train_fold = y_train[train_idxs]
-                model = self._get_model(n_classes, input_shape, mask)
+                model = self._get_model(
+                    n_classes,
+                    input_shape,
+                    mask=self.mask_value,
+                )
                 model.fit(
                     x_train_fold,
                     y_train_fold,
@@ -140,16 +158,15 @@ class TransformerModel(Model):
                 fold_no += 1
 
     def predict(self, data: Data) -> List[Any]:
-        x_data, _, _ = self._get_input_data(data)
+        x_data, _ = self._get_input_data(data)
         return self.model.predict(x_data)
 
     def _get_model(
-        self, n_classes: int, input_shape: tuple, mask: Union[np.ndarray, None] = None
+        self, n_classes: int, input_shape: tuple, mask: Any = None
     ) -> keras.Model:
         model = build_model(
             n_classes,
             input_shape,
-            input_mask=mask,
             head_size=self.head_size,
             num_heads=self.num_heads,
             ff_dim=self.ff_dim,
@@ -157,6 +174,7 @@ class TransformerModel(Model):
             mlp_units=self.mlp_units,
             mlp_dropout=self.mlp_dropout,
             dropout=self.dropout,
+            mask=mask,
         )
         model.compile(
             loss=self.loss,
@@ -165,9 +183,7 @@ class TransformerModel(Model):
         )
         return model
 
-    def _get_input_data(
-        self, data: Data
-    ) -> Tuple[np.ndarray, np.ndarray, Union[np.ndarray, None]]:
+    def _get_input_data(self, data: Data) -> Tuple[np.ndarray, np.ndarray]:
         """
         Process all the data and returns a x_data, y_data, mask readable
         by the transformer
@@ -175,20 +191,17 @@ class TransformerModel(Model):
         y_data = self._encode_labels(data)
         x_data = self._extract_raw_data(data)
         x_data = self._reshape_input(x_data)
-        # mask = self._mask_data(x_data)
-        return x_data, y_data, None
-
-    def _mask_data(self, x_data: np.ndarray) -> np.ndarray:
-        raise NotImplementedError
+        return x_data, y_data
 
     def _encode_labels(self, data: Data) -> np.ndarray:
         """Encode the labels"""
-        encoder = LabelEncoder()
-        encoder.fit(data.dataset.labels)
-        encoded_labels = encoder.transform(data.labels)
+        if self.encoder is None:
+            self.encoder = LabelEncoder()
+            self.encoder.fit(self.labels)
+        encoded_labels = self.encoder.transform(data.labels)
         assert isinstance(encoded_labels, np.ndarray)
 
-        classes = np.zeros((len(encoded_labels), len(encoder.classes_)))
+        classes = np.zeros((len(encoded_labels), len(self.encoder.classes_)))
         for i, label in enumerate(encoded_labels):
             classes[i][label] = 1
         return classes
@@ -206,10 +219,18 @@ class TransformerModel(Model):
         all_raw_data = np.zeros((len(raw_data), max_len, 3))
         for i, traj in enumerate(raw_data):
             traj = traj[:max_len]
-            all_raw_data[i, :, :] = 0  # TODO: check for masking
+            all_raw_data[i, :, :] = self.mask_value
             all_raw_data[i, : traj.shape[0]] = traj
         return all_raw_data
 
     def _reshape_input(self, x_data: np.ndarray) -> np.ndarray:
         """Reshapes the input data to be compatible with the transformer."""
         return x_data.reshape((x_data.shape[0], x_data.shape[1], x_data.shape[2], 1))
+
+    def evaluate(self, data: Data) -> Evaluation:
+        assert self.encoder is not None, "Encoder is not set."
+        x_data, _ = self._get_input_data(data)
+        preds = self.model.predict(x_data)
+        preds = [pred.argmax() for pred in preds]
+        evals = self.encoder.inverse_transform(preds)
+        return Evaluation.from_data(data, evals, self.summary)
